@@ -19,7 +19,7 @@ use crate::{
     elf::{FirmwareImageBuilder, FlashFrequency, FlashMode},
     error::Error,
     flasher::FlashSize,
-    Chip, Flasher, ImageFormatId, PartitionTable,
+    Chip, Flasher, ImageFormatId, InvalidPartitionTable, PartitionTable,
 };
 
 pub mod config;
@@ -65,6 +65,24 @@ pub struct FlashConfigOpts {
     pub flash_freq: Option<FlashFrequency>,
 }
 
+#[derive(Parser)]
+pub struct PartitionTableOpts {
+    /// Convert CSV parition table to binary representation
+    #[clap(long, required_unless_present_any = ["info", "to-csv"])]
+    to_binary: bool,
+    /// Convert binary partition table to CSV representation
+    #[clap(long, required_unless_present_any = ["info", "to-binary"])]
+    to_csv: bool,
+    /// Show information on partition table
+    #[clap(short, long, required_unless_present_any = ["to-binary", "to-csv"])]
+    info: bool,
+    /// Input partition table
+    partition_table: PathBuf,
+    /// Optional output file name, if unset will output to stdout
+    #[clap(short, long)]
+    output: Option<PathBuf>,
+}
+
 pub fn connect(opts: &ConnectOpts, config: &Config) -> Result<Flasher> {
     let port_info = get_serial_port_info(opts, config)?;
 
@@ -97,7 +115,7 @@ pub fn board_info(opts: ConnectOpts, config: Config) -> Result<()> {
 pub fn save_elf_as_image(
     chip: Chip,
     elf_data: &[u8],
-    path: PathBuf,
+    image_path: PathBuf,
     image_format: Option<ImageFormatId>,
     flash_mode: Option<FlashMode>,
     flash_size: Option<FlashSize>,
@@ -112,23 +130,11 @@ pub fn save_elf_as_image(
         .flash_freq(flash_freq)
         .build()?;
 
-    let flash_image = chip.get_flash_image(&image, None, None, image_format, None)?;
-    let parts: Vec<_> = flash_image.ota_segments().collect();
-
-    match parts.as_slice() {
-        [single] => fs::write(&path, &single.data).into_diagnostic()?,
-        parts => {
-            for part in parts {
-                let part_path = format!("{:#x}_{}", part.addr, path.display());
-                fs::write(part_path, &part.data).into_diagnostic()?
-            }
-        }
-    }
-
-    // merge_bin is TRUE
-    // merge bootloader, partition table and app binaries
-    // basic functionality, only merge 3 binaries
     if merge {
+        // merge_bin is TRUE
+        // merge bootloader, partition table and app binaries
+        // basic functionality, only merge 3 binaries
+
         // If the '-B' option is provided, load the bootloader binary file at the
         // specified path.
         let bootloader = if let Some(bootloader_path) = bootloader_path {
@@ -161,19 +167,11 @@ pub fn save_elf_as_image(
         let image =
             chip.get_flash_image(&image, bootloader, partition_table, image_format, None)?;
 
-        let merged_bin = format!(
-            "merged_{path}",
-            path = &path.to_str().unwrap_or("merged_bins.bin")
-        );
-
-        if Path::new(&merged_bin).exists() {
-            fs::remove_file(&merged_bin).into_diagnostic()?;
-        }
-
         let mut file = fs::OpenOptions::new()
             .write(true)
-            .create_new(true)
-            .open(merged_bin)
+            .truncate(true)
+            .create(true)
+            .open(image_path)
             .into_diagnostic()?;
 
         for segment in image.flash_segments() {
@@ -193,9 +191,21 @@ pub fn save_elf_as_image(
                 - file.metadata().into_diagnostic()?.len() as usize
         ];
         file.write_all(&padding_bytes).into_diagnostic()?;
+    } else {
+        let flash_image = chip.get_flash_image(&image, None, None, image_format, None)?;
+        let parts: Vec<_> = flash_image.ota_segments().collect();
+
+        match parts.as_slice() {
+            [single] => fs::write(&image_path, &single.data).into_diagnostic()?,
+            parts => {
+                for part in parts {
+                    let part_path = format!("{:#x}_{}", part.addr, image_path.display());
+                    fs::write(part_path, &part.data).into_diagnostic()?
+                }
+            }
+        }
     }
 
-    println!("The final merge binary was created successfully.");
     Ok(())
 }
 
@@ -251,3 +261,50 @@ pub fn flash_elf_image(
 
     Ok(())
 }
+
+pub fn partition_table(opts: PartitionTableOpts) -> Result<()> {
+    if opts.to_binary {
+        let input = fs::read(&opts.partition_table).into_diagnostic()?;
+        let part_table = PartitionTable::try_from_str(String::from_utf8(input).into_diagnostic()?)
+            .into_diagnostic()?;
+
+        // Use either stdout or a file if provided for the output.
+        let mut writer: Box<dyn Write> = if let Some(output) = opts.output {
+            Box::new(fs::File::create(output).into_diagnostic()?)
+        } else {
+            Box::new(std::io::stdout())
+        };
+        part_table.save_bin(&mut writer).into_diagnostic()?;
+    } else if opts.to_csv {
+        let input = fs::read(&opts.partition_table).into_diagnostic()?;
+        let part_table = PartitionTable::try_from_bytes(input).into_diagnostic()?;
+
+        // Use either stdout or a file if provided for the output.
+        let mut writer: Box<dyn Write> = if let Some(output) = opts.output {
+            Box::new(fs::File::create(output).into_diagnostic()?)
+        } else {
+            Box::new(std::io::stdout())
+        };
+        part_table.save_csv(&mut writer).into_diagnostic()?;
+    } else if opts.info {
+        let input = fs::read(&opts.partition_table).into_diagnostic()?;
+
+        // Try getting the partition table from either the csv or the binary representation and
+        // fail otherwise.
+        let part_table = if let Ok(part_table) =
+            PartitionTable::try_from_bytes(input.clone()).into_diagnostic()
+        {
+            part_table
+        } else if let Ok(part_table) =
+            PartitionTable::try_from_str(String::from_utf8(input).into_diagnostic()?)
+        {
+            part_table
+        } else {
+            return Err((InvalidPartitionTable {}).into());
+        };
+
+        part_table.pretty_print();
+    }
+
+    Ok(())
+}
